@@ -37,7 +37,7 @@
 #'   This argument is passed by expression and supports
 #'   [quasiquotation][rlang::quasiquotation] (you can unquote strings
 #'   and symbols). The name is captured from the expression with
-#'   [rlang::quo_name()] (note that this kind of interface where
+#'   [rlang::ensym()] (note that this kind of interface where
 #'   symbols do not represent actual objects is now discouraged in the
 #'   tidyverse; we support it here for backward compatibility).
 #' @param ... A selection of columns. If empty, all variables are
@@ -46,10 +46,10 @@
 #'   more options, see the [dplyr::select()] documentation. See also
 #'   the section on selection rules below.
 #' @param na.rm If `TRUE`, will remove rows from output where the
-#'   value column in `NA`.
+#'   value column is `NA`.
 #' @param convert If `TRUE` will automatically run
 #'   [type.convert()] on the key column. This is useful if the column
-#'   names are actually numeric, integer, or logical.
+#'   types are actually numeric, integer, or logical.
 #' @param factor_key If `FALSE`, the default, the key values will be
 #'   stored as a character vector. If `TRUE`, will be stored as a factor,
 #'   which preserves the original ordering of the columns.
@@ -65,16 +65,16 @@
 #'   Z = rnorm(10, 0, 4)
 #' )
 #'
-#' gather(stocks, stock, price, -time)
-#' stocks %>% gather(stock, price, -time)
+#' gather(stocks, "stock", "price", -time)
+#' stocks %>% gather("stock", "price", -time)
 #'
 #' # get first observation for each Species in iris data -- base R
 #' mini_iris <- iris[c(1, 51, 101), ]
 #' # gather Sepal.Length, Sepal.Width, Petal.Length, Petal.Width
-#' gather(mini_iris, key = flower_att, value = measurement,
+#' gather(mini_iris, key = "flower_att", value = "measurement",
 #'        Sepal.Length, Sepal.Width, Petal.Length, Petal.Width)
 #' # same result but less verbose
-#' gather(mini_iris, key = flower_att, value = measurement, -Species)
+#' gather(mini_iris, key = "flower_att", value = "measurement", -Species)
 #'
 #' # repeat iris example using dplyr and the pipe operator
 #' library(dplyr)
@@ -82,7 +82,7 @@
 #'   iris %>%
 #'   group_by(Species) %>%
 #'   slice(1)
-#' mini_iris %>% gather(key = flower_att, value = measurement, -Species)
+#' mini_iris %>% gather(key = "flower_att", value = "measurement", -Species)
 gather <- function(data, key = "key", value = "value", ...,
                    na.rm = FALSE, convert = FALSE, factor_key = FALSE) {
   UseMethod("gather")
@@ -91,8 +91,8 @@ gather <- function(data, key = "key", value = "value", ...,
 gather.data.frame <- function(data, key = "key", value = "value", ...,
                               na.rm = FALSE, convert = FALSE,
                               factor_key = FALSE) {
-  key_var <- quo_name(enexpr(key))
-  value_var <- quo_name(enexpr(value))
+  key_var <- as_string(ensym2(key))
+  value_var <- as_string(ensym2(value))
 
   quos <- quos(...)
   if (is_empty(quos)) {
