@@ -12,7 +12,7 @@
 #'   This argument is passed by expression and supports
 #'   [quasiquotation][rlang::quasiquotation] (you can unquote strings
 #'   and symbols). The name is captured from the expression with
-#'   [rlang::quo_name()] (note that this kind of interface where
+#'   [rlang::ensym()] (note that this kind of interface where
 #'   symbols do not represent actual objects is now discouraged in the
 #'   tidyverse; we support it here for backward compatibility).
 #' @seealso [unnest()] for the inverse operation.
@@ -33,17 +33,13 @@
 nest <- function(data, ..., .key = "data") {
   UseMethod("nest")
 }
+#' @importFrom utils packageVersion
 #' @export
-nest.default <- function(data, ..., .key = "data") {
-  key_col <- compat_as_lazy(enquo(.key))
-  nest_cols <- compat_as_lazy_dots(...)
-  nest_(data, key_col = key_col, nest_cols = nest_cols)
-}
-#' @export
-nest.data.frame <- function(data, ..., .key = "data") {
-  key_var <- quo_name(enexpr(.key))
+nest.tbl_df <- function(data, ..., .key = "data") {
 
+  key_var   <- as_string(ensym2(.key))
   nest_vars <- unname(tidyselect::vars_select(names(data), ...))
+
   if (is_empty(nest_vars)) {
     nest_vars <- names(data)
   }
@@ -61,27 +57,28 @@ nest.data.frame <- function(data, ..., .key = "data") {
   }
 
   out <- dplyr::select(data, !!! syms(group_vars))
-  out <- dplyr::distinct(out)
 
-  idx <- dplyr::group_indices(data, !!! syms(group_vars))
+  if (packageVersion("dplyr") < "0.8.0") {
+    idx <- dplyr::group_indices(data, !!! syms(group_vars))
+  } else {
+    idx <- dplyr::group_indices(data, !!! syms(group_vars), .drop = TRUE)
+  }
+
+  representatives <- which(!duplicated(idx))
+
+  out <- dplyr::slice(out, representatives)
   out[[key_var]] <- unname(split(data[nest_vars], idx))[unique(idx)]
-
   out
 }
 
+#' @export
+nest.data.frame <- function(data, ..., .key = "data") {
+  .key <- enquo(.key)
 
-#' @rdname deprecated-se
-#' @inheritParams nest
-#' @param key_col Name of the column that will contain the nested data frames.
-#' @param nest_cols Character vector of columns to nest.
-#' @keywords internal
-#' @export
-nest_ <- function(data, key_col, nest_cols = character()) {
-  UseMethod("nest_")
-}
-#' @export
-nest_.data.frame <- function(data, key_col, nest_cols = character()) {
-  key_col <- sym(key_col)
-  nest_cols <- syms(nest_cols)
-  nest(data, .key = !! key_col, !!! nest_cols)
+  # Don't transform subclasses
+  if (identical(class(data), "data.frame")) {
+    data <- tibble::as_tibble(data)
+  }
+
+  nest.tbl_df(data, ..., .key = !! .key)
 }
