@@ -37,7 +37,7 @@
 #'
 #' # Spread and gather are complements
 #' df <- data.frame(x = c("a", "b"), y = c(3, 4), z = c(5, 6))
-#' df %>% spread(x, y) %>% gather(x, y, a:b, na.rm = TRUE)
+#' df %>% spread(x, y) %>% gather("x", "y", a:b, na.rm = TRUE)
 #'
 #' # Use 'convert = TRUE' to produce variables of mixed type
 #' df <- data.frame(row = rep(c(1, 51), each = 3),
@@ -50,20 +50,6 @@ spread <- function(data, key, value, fill = NA, convert = FALSE,
   UseMethod("spread")
 }
 #' @export
-spread.default <- function(data, key, value, fill = NA, convert = FALSE,
-                           drop = TRUE, sep = NULL) {
-  key <- compat_as_lazy(enquo(key))
-  value <- compat_as_lazy(enquo(value))
-  spread_(data,
-    key_col = key,
-    value_col = value,
-    fill = fill,
-    convert = convert,
-    drop = drop,
-    sep = sep
-  )
-}
-#' @export
 spread.data.frame <- function(data, key, value, fill = NA, convert = FALSE,
                               drop = TRUE, sep = NULL) {
   key_var <- tidyselect::vars_pull(names(data), !! enquo(key))
@@ -74,7 +60,7 @@ spread.data.frame <- function(data, key, value, fill = NA, convert = FALSE,
   col_labels <- split_labels(col, col_id, drop = drop)
 
   rows <- data[setdiff(names(data), c(key_var, value_var))]
-  if (length(rows) == 0) {
+  if (ncol(rows) == 0 && nrow(rows) > 0) {
     # Special case when there's only one row
     row_id <- structure(1L, n = 1L)
     row_labels <- as.data.frame(matrix(nrow = 1, ncol = 0))
@@ -91,9 +77,16 @@ spread.data.frame <- function(data, key, value, fill = NA, convert = FALSE,
     groups <- split(seq_along(overall), overall)
     groups <- groups[map_int(groups, length) > 1]
 
-    str <- map_chr(groups, function(x) paste0("(", paste0(x, collapse = ", "), ")"))
-    rows <- paste(str, collapse = ", ")
-    abort(glue("Duplicate identifiers for rows {rows}"))
+    shared <- sum(map_int(groups, length))
+
+    str <- map_chr(groups, function(x) paste0(x, collapse = ", "))
+    rows <- paste0(paste0("* ", str, "\n"), collapse = "")
+    abort(glue(
+      "Each row of output must be identified by a unique combination of keys.",
+      "\nKeys are shared for {shared} rows:",
+      "\n{rows}",
+      "Do you need to create unique ID with tibble::rowid_to_column()?"
+    ))
   }
 
   # Add in missing values, if necessary
@@ -129,7 +122,12 @@ col_names <- function(x, sep = NULL) {
   names <- as.character(x[[1]])
 
   if (is_null(sep)) {
-    ifelse(are_na(names), "<NA>", names)
+    if (length(names) == 0) {
+      # ifelse will return logical()
+      character()
+    } else {
+      ifelse(are_na(names), "<NA>", names)
+    }
   } else {
     paste(names(x)[[1]], names, sep = sep)
   }
@@ -145,7 +143,9 @@ split_labels <- function(df, id, drop = TRUE) {
 
   if (drop) {
     representative <- match(sort(unique(id)), id)
-    df[representative, , drop = FALSE]
+    out <- df[representative, , drop = FALSE]
+    rownames(out) <- NULL
+    out
   } else {
     unique_values <- map(df, ulevels)
     rev(expand.grid(rev(unique_values), stringsAsFactors = FALSE))
@@ -153,35 +153,13 @@ split_labels <- function(df, id, drop = TRUE) {
 }
 ulevels <- function(x) {
   if (is.factor(x)) {
+    orig_levs <- levels(x)
     x <- addNA(x, ifany = TRUE)
     levs <- levels(x)
-    factor(levs, levels = levs, ordered = is.ordered(x))
+    factor(levs, levels = orig_levs, ordered = is.ordered(x), exclude = NULL)
+  } else if (is.list(x)) {
+    unique(x)
   } else {
-    sort(unique(x))
+    sort(unique(x), na.last = TRUE)
   }
 }
-
-
-#' @rdname deprecated-se
-#' @inheritParams spread
-#' @param key_col,value_col Strings giving names of key and value cols.
-#' @export
-spread_ <- function(data, key_col, value_col, fill = NA, convert = FALSE,
-                    drop = TRUE, sep = NULL) {
-  UseMethod("spread_")
-}
-#' @export
-spread_.data.frame <- function(data, key_col, value_col, fill = NA,
-                               convert = FALSE, drop = TRUE, sep = NULL) {
-  key_col <- sym(key_col)
-  value_col <- sym(value_col)
-
-  spread(data,
-    key = !! key_col,
-    value = !! value_col,
-    fill = fill,
-    convert = convert,
-    drop = drop,
-    sep = sep
-  )
-}
