@@ -10,7 +10,7 @@
 #'   This argument is passed by expression and supports
 #'   [quasiquotation][rlang::quasiquotation] (you can unquote strings
 #'   and symbols). The name is captured from the expression with
-#'   [rlang::quo_name()] (note that this kind of interface where
+#'   [rlang::ensym()] (note that this kind of interface where
 #'   symbols do not represent actual objects is now discouraged in the
 #'   tidyverse; we support it here for backward compatibility).
 #' @param sep Separator to use between values.
@@ -23,21 +23,20 @@
 #'
 #' # Separate is the complement of unite
 #' mtcars %>%
-#'   unite(vs_am, vs, am) %>%
+#'   unite("vs_am", vs, am) %>%
 #'   separate(vs_am, c("vs", "am"))
 unite <- function(data, col, ..., sep = "_", remove = TRUE) {
   UseMethod("unite")
 }
 #' @export
-unite.default <- function(data, col, ..., sep = "_", remove = TRUE) {
-  col <- compat_as_lazy(enquo(col))
-  from <- compat_as_lazy_dots(...)
-  unite_(data, col, from, sep = sep, remove = remove)
-}
-#' @export
 unite.data.frame <- function(data, col, ..., sep = "_", remove = TRUE) {
-  var <- quo_name(enquo(col))
-  from_vars <- tidyselect::vars_select(colnames(data), ...)
+  var <- as_string(ensym2(col))
+
+  if (dots_n(...) == 0) {
+    from_vars <- colnames(data)
+  } else {
+    from_vars <- tidyselect::vars_select(colnames(data), ...)
+  }
 
   out <- data
   if (remove) {
@@ -47,21 +46,6 @@ unite.data.frame <- function(data, col, ..., sep = "_", remove = TRUE) {
   first_pos <- which(names(data) %in% from_vars)[1]
   united <- invoke(paste, c(data[from_vars], list(sep = sep)))
 
-  out <- append_col(out, united, var, after = first_pos - 1)
+  out <- append_col(out, united, var, after = first_pos - 1L)
   reconstruct_tibble(data, out, if (remove) from_vars)
 }
-
-
-#' @rdname deprecated-se
-#' @inheritParams unite
-#' @param from Names of existing columns as character vector
-#' @export
-unite_ <- function(data, col, from, sep = "_", remove = TRUE) {
-  UseMethod("unite_")
-}
-#' @export
-unite_.data.frame <- function(data, col, from, sep = "_", remove = TRUE) {
-  col <- compat_lazy(col, caller_env())
-  from <- syms(from)
-  unite(data, !! col, !!! from, sep = sep, remove = remove)
-}
