@@ -24,7 +24,7 @@ test_that("unnesting row binds data frames", {
   expect_equal(unnest(df)$x, 1:10)
 })
 
-test_that("can unnested lists", {
+test_that("can unnest nested lists", {
   df <- tibble(
     x = 1:2,
     y = list(list("a"), list("b"))
@@ -33,9 +33,21 @@ test_that("can unnested lists", {
   expect_identical(rs, tibble(x = 1:2, y = list("a", "b")))
 })
 
+test_that("can unnest mixture of name and unnamed lists of same length", {
+  df <- tibble(
+    x = c("a"),
+    y = list(y = 1:2),
+    z = list(1:2)
+  )
+  expect_identical(unnest(df), tibble(x = c("a","a"), y = c(1:2), z = c(1:2)))
+})
+
 test_that("elements must all be of same type", {
   df <- tibble(x = list(1, "a"))
-  expect_error(unnest(df), "(incompatible type)|(numeric to character)|(character to numeric)")
+  expect_error(
+    unnest(df),
+    "(incompatible type)|(numeric to character)|(character to numeric)"
+  )
 })
 
 test_that("can't combine vectors and data frames", {
@@ -167,3 +179,14 @@ test_that("grouping is preserved", {
   expect_equal(class(df), class(rs))
   expect_equal(dplyr::groups(df), dplyr::groups(rs))
 })
+
+test_that("unnesting zero row column preserves names", {
+  df <- tibble(a = character(), b = character())
+  expect_equal(df %>% unnest(b), tibble(b = character(), a = character()))
+})
+
+test_that("unnest() recognize ptype", {
+  tbl <- tibble(x = integer(), y = structure(list(), ptype = double()))
+  res <- unnest(tbl)
+  expect_equal(res, tibble(x = integer(), y = double()))
+})
